@@ -6,6 +6,7 @@
 #
 """Module for tasks related to doing work from a queue"""
 from six.moves import queue
+from sparts.counters import counter, samples, SampleType, CallbackCounter
 from sparts.sparts import option
 from sparts.vtask import VTask, ExecuteContext, TryLater
 
@@ -20,6 +21,12 @@ class QueueTask(VTask):
                      help='Number of threads to spawn to work on items from '
                           'its queue. [%(default)s]')
 
+    execute_duration_ms = samples(windows=[60, 240],
+       types=[SampleType.AVG, SampleType.MAX, SampleType.MIN])
+    n_trylater = counter()
+    n_completed = counter()
+    n_unhandled = counter()
+
     def execute(self, item, context):
         """Implement this in your QueueTask subclasses"""
         raise NotImplementedError()
@@ -27,6 +34,8 @@ class QueueTask(VTask):
     def initTask(self):
         super(QueueTask, self).initTask()
         self.queue = queue.Queue(maxsize=self.max_items)
+        self.counters['queue_depth'] = \
+            CallbackCounter(lambda: self.queue.qsize())
         self._shutdown_sentinel = object()
 
     def stop(self):
@@ -43,30 +52,29 @@ class QueueTask(VTask):
             except queue.Empty:
                 continue
 
+            # Create an ExecuteContext if we didn't have one
             if isinstance(item, ExecuteContext):
                 context = item
                 item = context.item
             else:
                 context = ExecuteContext(item=item)
+
             try:
+                context.start()
                 result = self.execute(item, context)
-                if context.deferred is not None:
-                    context.deferred.callback(result)
+                self.n_completed.increment()
+                self.execute_duration_ms.add(context.elapsed * 1000.0)
+                context.set_result(result)
             except TryLater:
+                self.n_trylater.increment()
                 context.attempt += 1
                 self.queue.put(context)
             except Exception as ex:
-                if context.deferred is not None:
-                    self.unhandled = None
-                    context.deferred.addErrback(self.unhandledErrback)
-                    context.deferred.errback(ex)
-                    if self.unhandled is not None:
-                        self.unhandled.raiseException()
-                else:
+                self.n_unhandled.increment()
+                self.execute_duration_ms.add(context.elapsed * 1000.0)
+                handled = context.set_exception(ex)
+                if not handled:
                     raise
+
             finally:
                 self.queue.task_done()
-
-    def unhandledErrback(self, error):
-        self.unhandled = error
-        return None
