@@ -29,6 +29,22 @@ class ThriftHandlerTask(VTask):
     def initTask(self):
         super(ThriftHandlerTask, self).initTask()
         assert self.MODULE is not None
+        self._verifyInterface()
+
+    def _verifyInterface(self):
+        iface = self.MODULE.Iface
+        missing_methods = []
+        for k in dir(iface):
+            v = getattr(iface, k, None)
+            if not callable(v) or k.startswith('_'):
+                continue
+            v2 = getattr(self, k, None)
+            if v2 is None or not callable(v):
+                missing_methods.append(k)
+
+        if missing_methods:
+            raise TypeError("%s is missing the following methods: %s" %
+                (self.__class__.__name__, missing_methods))
 
     def _makeProcessor(self):
         return self.MODULE.Processor(self)
@@ -120,14 +136,38 @@ class NBServerTask(ThriftServerTask):
         super(NBServerTask, self).initTask()
 
         self._stopped = False
-        self.socket = TServerSocket(self.host, self.port)
+
+        # Construct TServerSocket this way for compatibility with fbthrift
+        self.socket = TServerSocket(port=self.port)
+        self.socket.host = self.host
+
         self.server = TNonblockingServer(self.processor, self.socket,
                                          threads=self.num_threads)
         self.server.prepare()
-        self.bound_host, self.bound_port = \
-            self.server.socket.handle.getsockname()
-        self.logger.info("%s Server Started on %s:%s",
-                         self.name, self.bound_host, self.bound_port)
+
+        self.bound_addrs = []
+        for handle in self._get_socket_handles(self.server.socket):
+            addrinfo = handle.getsockname()
+            self.bound_host, self.bound_port = addrinfo[0:2]
+            self.logger.info("%s Server Started on %s", self.name,
+                self._fmt_hostport(self.bound_host, self.bound_port))
+
+    def _get_socket_handles(self, tsocket):
+        """Helper to retrieve the socket objects for a given TServerSocket"""
+        handle = getattr(tsocket, 'handle', None)
+        if handle is not None:
+            return [tsocket.handle]
+
+        # Some TServerSocket implementations support multiple handles per
+        # TServerSocket (e.g., to support binding v4 and v6 without
+        # v4-mapped addresses
+        return tsocket.handles.values()
+
+    def _fmt_hostport(self, host, port):
+        if ':' in host:
+            return '[%s]:%d' % (host, port)
+        else:
+            return '%s:%d' % (host, port)
 
     def stop(self):
         """Overridden to tell the thrift server to shutdown asynchronously"""
